@@ -3,7 +3,7 @@
  * rum_ts_utils.c
  *		various text-search functions
  *
- * Portions Copyright (c) 2015-2016, Postgres Professional
+ * Portions Copyright (c) 2015-2019, Postgres Professional
  * Portions Copyright (c) 1996-2016, PostgreSQL Global Development Group
  *
  *-------------------------------------------------------------------------
@@ -20,6 +20,9 @@
 #include "tsearch/ts_utils.h"
 #include "utils/array.h"
 #include "utils/builtins.h"
+#if PG_VERSION_NUM >= 120000
+#include "utils/float.h"
+#endif
 #include "utils/typcache.h"
 
 #include "rum.h"
@@ -43,6 +46,9 @@ PG_FUNCTION_INFO_V1(rum_tsquery_distance);
 PG_FUNCTION_INFO_V1(rum_ts_distance_tt);
 PG_FUNCTION_INFO_V1(rum_ts_distance_ttf);
 PG_FUNCTION_INFO_V1(rum_ts_distance_td);
+PG_FUNCTION_INFO_V1(rum_ts_score_tt);
+PG_FUNCTION_INFO_V1(rum_ts_score_ttf);
+PG_FUNCTION_INFO_V1(rum_ts_score_td);
 PG_FUNCTION_INFO_V1(rum_ts_join_pos);
 
 PG_FUNCTION_INFO_V1(tsquery_to_distance_query);
@@ -1236,7 +1242,7 @@ calc_score_docr(float4 *arrdata, DocRepresentation *doc, uint32 doclen,
 		int			new_cover_key = 0;
 		int			nitems = 0;
 
-		while (ptr <= ext.end)
+		while (ptr && ptr <= ext.end)
 		{
 			InvSum += arrdata[ptr->wclass];
 			/* SK: Quick and dirty hash key. Hope collisions will be not too frequent. */
@@ -1470,15 +1476,9 @@ rum_ts_distance_ttf(PG_FUNCTION_ARGS)
 		PG_RETURN_FLOAT4(1.0 / res);
 }
 
-/*
- * Implementation of <=> operator. Uses specified normalization method.
- */
-Datum
-rum_ts_distance_td(PG_FUNCTION_ARGS)
+static float4
+calc_score_parse_opt(TSVector txt, HeapTupleHeader d)
 {
-	TSVector	txt = PG_GETARG_TSVECTOR(0);
-	HeapTupleHeader d = PG_GETARG_HEAPTUPLEHEADER(1);
-
 	Oid			tupType = HeapTupleHeaderGetTypeId(d);
 	int32		tupTypmod = HeapTupleHeaderGetTypMod(d);
 	TupleDesc	tupdesc = lookup_rowtype_tupdesc(tupType, tupTypmod);
@@ -1498,8 +1498,6 @@ rum_ts_distance_td(PG_FUNCTION_ARGS)
 	if (isnull)
 	{
 		ReleaseTupleDesc(tupdesc);
-		PG_FREE_IF_COPY(txt, 0);
-		PG_FREE_IF_COPY(d, 1);
 		elog(ERROR, "NULL query value is not allowed");
 	}
 
@@ -1510,6 +1508,22 @@ rum_ts_distance_td(PG_FUNCTION_ARGS)
 	res = calc_score(weights, txt, query, method);
 
 	ReleaseTupleDesc(tupdesc);
+
+	return res;
+}
+
+/*
+ * Implementation of <=> operator. Uses specified normalization method.
+ */
+Datum
+rum_ts_distance_td(PG_FUNCTION_ARGS)
+{
+	TSVector	txt = PG_GETARG_TSVECTOR(0);
+	HeapTupleHeader d = PG_GETARG_HEAPTUPLEHEADER(1);
+	float4		res;
+
+	res = calc_score_parse_opt(txt, d);
+
 	PG_FREE_IF_COPY(txt, 0);
 	PG_FREE_IF_COPY(d, 1);
 
@@ -1519,6 +1533,61 @@ rum_ts_distance_td(PG_FUNCTION_ARGS)
 		PG_RETURN_FLOAT4(1.0 / res);
 }
 
+/*
+ * Calculate score (inverted distance). Uses default normalization method.
+ */
+Datum
+rum_ts_score_tt(PG_FUNCTION_ARGS)
+{
+	TSVector	txt = PG_GETARG_TSVECTOR(0);
+	TSQuery		query = PG_GETARG_TSQUERY(1);
+	float4		res;
+
+	res = calc_score(weights, txt, query, DEF_NORM_METHOD);
+
+	PG_FREE_IF_COPY(txt, 0);
+	PG_FREE_IF_COPY(query, 1);
+
+	PG_RETURN_FLOAT4(res);
+}
+
+/*
+ * Calculate score (inverted distance). Uses specified normalization method.
+ */
+Datum
+rum_ts_score_ttf(PG_FUNCTION_ARGS)
+{
+	TSVector	txt = PG_GETARG_TSVECTOR(0);
+	TSQuery		query = PG_GETARG_TSQUERY(1);
+	int			method = PG_GETARG_INT32(2);
+	float4		res;
+
+	res = calc_score(weights, txt, query, method);
+
+	PG_FREE_IF_COPY(txt, 0);
+	PG_FREE_IF_COPY(query, 1);
+
+	PG_RETURN_FLOAT4(res);
+}
+
+/*
+ * Calculate score (inverted distance). Uses specified normalization method.
+ */
+Datum
+rum_ts_score_td(PG_FUNCTION_ARGS)
+{
+	TSVector	txt = PG_GETARG_TSVECTOR(0);
+	HeapTupleHeader d = PG_GETARG_HEAPTUPLEHEADER(1);
+	float4		res;
+
+	res = calc_score_parse_opt(txt, d);
+
+	PG_FREE_IF_COPY(txt, 0);
+	PG_FREE_IF_COPY(d, 1);
+
+	PG_RETURN_FLOAT4(res);
+}
+
 /*
  * Casts tsquery to rum_distance_query type.
  */
