@@ -4,7 +4,7 @@
  *	  routines for fast build of inverted index
  *
  *
- * Portions Copyright (c) 2015-2016, Postgres Professional
+ * Portions Copyright (c) 2015-2019, Postgres Professional
  * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
@@ -14,7 +14,6 @@
 #include "postgres.h"
 
 #include "utils/datum.h"
-#include "utils/memutils.h"
 
 #include "rum.h"
 
@@ -22,7 +21,7 @@
 #define DEF_NPTR	5			/* ItemPointer initial allocation quantum */
 
 /* PostgreSQL pre 10 has different names for this functions */
-#if PG_VERSION_NUM <= 100006
+#if PG_VERSION_NUM <= 100006 || PG_VERSION_NUM == 110000
 #define rbt_create(node_size, comparator, combiner, allocfunc, freefunc, arg) \
 	(rb_create(node_size, comparator, combiner, allocfunc, freefunc, arg))
 #define rbt_insert(rbt, data, isNew) \
@@ -167,6 +166,7 @@ rumInsertBAEntry(BuildAccumulator *accum,
 	eatmp.category = category;
 	/* temporarily set up single-entry itempointer list */
 	eatmp.list = &item;
+	memset(&item, 0, sizeof(item));
 	item.iptr = *heapptr;
 	item.addInfo = addInfo;
 	item.addInfoIsNull = addInfoIsNull;
@@ -280,7 +280,7 @@ qsortCompareRumItem(const void *a, const void *b, void *arg)
 void
 rumBeginBAScan(BuildAccumulator *accum)
 {
-#if PG_VERSION_NUM > 100006
+#if (PG_VERSION_NUM > 100006 && PG_VERSION_NUM < 110000) || PG_VERSION_NUM >= 110001
 	rbt_begin_iterate(accum->tree, LeftRightWalk, &accum->tree_walk);
 #elif PG_VERSION_NUM >= 100000
 	rb_begin_iterate(accum->tree, LeftRightWalk, &accum->tree_walk);
@@ -302,7 +302,7 @@ rumGetBAEntry(BuildAccumulator *accum,
 	RumEntryAccumulator *entry;
 	RumItem	   *list;
 
-#if PG_VERSION_NUM > 100006
+#if (PG_VERSION_NUM > 100006 && PG_VERSION_NUM < 110000) || PG_VERSION_NUM >= 110001
 	entry = (RumEntryAccumulator *) rbt_iterate(&accum->tree_walk);
 #elif PG_VERSION_NUM >= 100000
 	entry = (RumEntryAccumulator *) rb_iterate(&accum->tree_walk);
