@@ -4,7 +4,7 @@
  *	  page utilities routines for the postgres inverted index access method.
  *
  *
- * Portions Copyright (c) 2015-2016, Postgres Professional
+ * Portions Copyright (c) 2015-2019, Postgres Professional
  * Portions Copyright (c) 1996-2016, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
@@ -589,7 +589,7 @@ findInLeafPage(RumBtree btree, Page page, OffsetNumber *offset,
 		*iptrOut = item.iptr;
 
 		ptr = rumDataPageLeafRead(ptr, btree->entryAttnum, &item,
-								  btree->rumstate);
+								  false, btree->rumstate);
 
 		cmp = compareRumItem(btree->rumstate, btree->entryAttnum,
 							 &btree->items[btree->curitem], &item);
@@ -899,7 +899,8 @@ dataPlaceToPage(RumBtree btree, Page page, OffsetNumber off)
 			if (copyItemEmpty == true && off <= maxoff)
 			{
 				copyPtr = rumDataPageLeafRead(copyPtr, btree->entryAttnum,
-											  &copyItem, btree->rumstate);
+											  &copyItem, false,
+											  btree->rumstate);
 				copyItemEmpty = false;
 			}
 
@@ -1056,6 +1057,7 @@ dataSplitPageLeaf(RumBtree btree, Buffer lbuf, Buffer rbuf,
 	int			maxItemIndex = btree->curitem;
 	static char lpageCopy[BLCKSZ];
 
+	memset(&item, 0, sizeof(item));
 	dataPrepareData(btree, newlPage, off);
 	maxoff = RumPageGetOpaque(newlPage)->maxoff;
 
@@ -1090,7 +1092,7 @@ dataSplitPageLeaf(RumBtree btree, Buffer lbuf, Buffer rbuf,
 
 		prevIptr = item.iptr;
 		copyPtr = rumDataPageLeafRead(copyPtr, btree->entryAttnum, &item,
-									  btree->rumstate);
+									  false, btree->rumstate);
 
 		prevTotalsize = totalsize;
 		totalsize = rumCheckPlaceToDataPageLeaf(btree->entryAttnum,
@@ -1168,7 +1170,7 @@ dataSplitPageLeaf(RumBtree btree, Buffer lbuf, Buffer rbuf,
 		}
 
 		copyPtr = rumDataPageLeafRead(copyPtr, btree->entryAttnum, &item,
-									  btree->rumstate);
+									  false, btree->rumstate);
 
 		curItem = item;
 		ptr = rumPlaceToDataPageLeaf(ptr, btree->entryAttnum, &item,
@@ -1280,8 +1282,8 @@ dataSplitPageInternal(RumBtree btree, Buffer lbuf, Buffer rbuf,
 
 	PostingItemSetBlockNumber(&(btree->pitem), BufferGetBlockNumber(lbuf));
 	if (RumPageIsLeaf(newlPage))
-		btree->pitem.item.iptr = *(ItemPointerData *) RumDataPageGetItem(newlPage,
-										 RumPageGetOpaque(newlPage)->maxoff);
+		btree->pitem.item.iptr = ((PostingItem *) RumDataPageGetItem(newlPage,
+								   RumPageGetOpaque(newlPage)->maxoff))->item.iptr;
 	else
 		btree->pitem.item = ((PostingItem *) RumDataPageGetItem(newlPage,
 								   RumPageGetOpaque(newlPage)->maxoff))->item;
@@ -1350,7 +1352,7 @@ updateItemIndexes(Page page, OffsetNumber attnum, RumState * rumstate)
 			}
 			j++;
 		}
-		ptr = rumDataPageLeafRead(ptr, attnum, &item, rumstate);
+		ptr = rumDataPageLeafRead(ptr, attnum, &item, false, rumstate);
 	}
 	/* Fill rest of page indexes with InvalidOffsetNumber if any */
 	for (; j < RumDataLeafIndexCount; j++)
@@ -1433,10 +1435,12 @@ rumDataFillRoot(RumBtree btree, Buffer root, Buffer lbuf, Buffer rbuf,
 	PostingItem li,
 				ri;
 
+	memset(&li, 0, sizeof(PostingItem));
 	li.item = *RumDataPageGetRightBound(lpage);
 	PostingItemSetBlockNumber(&li, BufferGetBlockNumber(lbuf));
 	RumDataPageAddItem(page, &li, InvalidOffsetNumber);
 
+	memset(&ri, 0, sizeof(PostingItem));
 	ri.item = *RumDataPageGetRightBound(rpage);
 	PostingItemSetBlockNumber(&ri, BufferGetBlockNumber(rbuf));
 	RumDataPageAddItem(page, &ri, InvalidOffsetNumber);
