@@ -4,7 +4,7 @@
  *	  fetch tuples from a RUM scan.
  *
  *
- * Portions Copyright (c) 2015-2016, Postgres Professional
+ * Portions Copyright (c) 2015-2019, Postgres Professional
  * Portions Copyright (c) 1996-2016, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
@@ -22,7 +22,6 @@
 #if PG_VERSION_NUM >= 120000
 #include "utils/float.h"
 #endif
-#include "utils/memutils.h"
 
 #include "rum.h"
 
@@ -258,7 +257,8 @@ scanPostingTree(Relation index, RumScanEntry scanEntry,
 			ptr = RumDataPageGetData(page);
 			for (i = FirstOffsetNumber; i <= maxoff; i++)
 			{
-				ptr = rumDataPageLeafRead(ptr, attnum, &item.item, rumstate);
+				ptr = rumDataPageLeafRead(ptr, attnum, &item.item, false,
+										  rumstate);
 				SCAN_ITEM_PUT_KEY(scanEntry, item, idatum, icategory);
 				rum_tuplesort_putrumitem(scanEntry->matchSortstate, &item);
 			}
@@ -468,7 +468,7 @@ collectMatchBitmap(RumBtreeData * btree, RumBtreeStack * stack,
 			for (i = 0; i < RumGetNPosting(itup); i++)
 			{
 				ptr = rumDataPageLeafRead(ptr, scanEntry->attnum, &item.item,
-										  rumstate);
+										  false, rumstate);
 				SCAN_ITEM_PUT_KEY(scanEntry, item, idatum, icategory);
 				rum_tuplesort_putrumitem(scanEntry->matchSortstate, &item);
 			}
@@ -674,7 +674,8 @@ restartScanEntry:
 
 			for (i = FirstOffsetNumber; i <= maxoff; i = OffsetNumberNext(i))
 			{
-				ptr = rumDataPageLeafRead(ptr, entry->attnum, &item, rumstate);
+				ptr = rumDataPageLeafRead(ptr, entry->attnum, &item, true,
+										  rumstate);
 				entry->list[i - FirstOffsetNumber] = item;
 			}
 
@@ -689,7 +690,7 @@ restartScanEntry:
 			entry->predictNumberResult = entry->nlist;
 			entry->list = (RumItem *) palloc(sizeof(RumItem) * entry->nlist);
 
-			rumReadTuple(rumstate, entry->attnum, itup, entry->list);
+			rumReadTuple(rumstate, entry->attnum, itup, entry->list, true);
 			entry->isFinished = setListPositionScanEntry(rumstate, entry);
 			if (!entry->isFinished)
 				entry->curItem = entry->list[entry->offset];
@@ -935,7 +936,8 @@ entryGetNextItem(RumState * rumstate, RumScanEntry entry, Snapshot snapshot)
 
 			for (i = FirstOffsetNumber; i <= maxoff; i = OffsetNumberNext(i))
 			{
-				ptr = rumDataPageLeafRead(ptr, entry->attnum, &item, rumstate);
+				ptr = rumDataPageLeafRead(ptr, entry->attnum, &item, true,
+										  rumstate);
 				entry->list[i - FirstOffsetNumber] = item;
 
 				if (searchBorder)
@@ -1091,7 +1093,8 @@ entryGetNextItemList(RumState * rumstate, RumScanEntry entry, Snapshot snapshot)
 
 		for (i = FirstOffsetNumber; i <= maxoff; i = OffsetNumberNext(i))
 		{
-			ptr = rumDataPageLeafRead(ptr, entry->attnum, &item, rumstate);
+			ptr = rumDataPageLeafRead(ptr, entry->attnum, &item, true,
+									  rumstate);
 			entry->list[i - FirstOffsetNumber] = item;
 		}
 
@@ -1104,11 +1107,11 @@ entryGetNextItemList(RumState * rumstate, RumScanEntry entry, Snapshot snapshot)
 		entry->predictNumberResult = entry->nlist;
 		entry->list = (RumItem *) palloc(sizeof(RumItem) * entry->nlist);
 
-		rumReadTuple(rumstate, entry->attnum, itup, entry->list);
+		rumReadTuple(rumstate, entry->attnum, itup, entry->list, true);
 		entry->isFinished = setListPositionScanEntry(rumstate, entry);
 	}
 
-	Assert(entry->nlist > 0);
+	Assert(entry->nlist > 0 && entry->list);
 
 	entry->curItem = entry->list[entry->offset];
 	entry->offset += entry->scanDirection;
@@ -1165,7 +1168,7 @@ entryGetItem(RumState * rumstate, RumScanEntry entry, bool *nextEntryList, Snaps
 			if (!ItemPointerIsMin(&entry->collectRumItem.item.iptr))
 				collected = entry->collectRumItem;
 			else
-				ItemPointerSetMin(&collected.item.iptr);
+				MemSet(&collected, 0, sizeof(collected));
 
 			ItemPointerSetMin(&entry->curItem.iptr);
 
@@ -1659,7 +1662,8 @@ scanPage(RumState * rumstate, RumScanEntry entry, RumItem *item, bool equalOk)
 	bound = -1;
 	for (i = first; i <= maxoff; i++)
 	{
-		ptr = rumDataPageLeafRead(ptr, entry->attnum, &iter_item, rumstate);
+		ptr = rumDataPageLeafRead(ptr, entry->attnum, &iter_item, true,
+								  rumstate);
 		entry->list[i - first] = iter_item;
 
 		if (bound != -1)
@@ -2241,8 +2245,8 @@ insertScanItem(RumScanOpaque so, bool recheck)
 				j;
 
 	item = (RumSortItem *)
-		MemoryContextAlloc(rum_tuplesort_get_memorycontext(so->sortstate),
-						   RumSortItemSize(so->norderbys));
+		MemoryContextAllocZero(rum_tuplesort_get_memorycontext(so->sortstate),
+							   RumSortItemSize(so->norderbys));
 	item->iptr = so->item.iptr;
 	item->recheck = recheck;
 
@@ -2352,6 +2356,14 @@ rumgettuple(IndexScanDesc scan, ScanDirection direction)
 	RumSortItem *item;
 	bool		should_free;
 
+#if PG_VERSION_NUM >= 120000
+#define GET_SCAN_TID(scan)		((scan)->xs_heaptid)
+#define SET_SCAN_TID(scan, tid)	((scan)->xs_heaptid = (tid))
+#else
+#define GET_SCAN_TID(scan)		((scan)->xs_ctup.t_self)
+#define SET_SCAN_TID(scan, tid)	((scan)->xs_ctup.t_self = (tid))
+#endif
+
 	if (so->firstCall)
 	{
 		/*
@@ -2384,7 +2396,7 @@ rumgettuple(IndexScanDesc scan, ScanDirection direction)
 	{
 		if (scanGetItem(scan, &so->item, &so->item, &recheck))
 		{
-			scan->xs_ctup.t_self = so->item.iptr;
+			SET_SCAN_TID(scan, so->item.iptr);
 			scan->xs_recheck = recheck;
 			scan->xs_recheckorderby = false;
 
@@ -2406,7 +2418,7 @@ rumgettuple(IndexScanDesc scan, ScanDirection direction)
 		uint32		i,
 					j = 0;
 
-		if (rumCompareItemPointers(&scan->xs_ctup.t_self, &item->iptr) == 0)
+		if (rumCompareItemPointers(&GET_SCAN_TID(scan), &item->iptr) == 0)
 		{
 			if (should_free)
 				pfree(item);
@@ -2414,7 +2426,7 @@ rumgettuple(IndexScanDesc scan, ScanDirection direction)
 			continue;
 		}
 
-		scan->xs_ctup.t_self = item->iptr;
+		SET_SCAN_TID(scan, item->iptr);
 		scan->xs_recheck = item->recheck;
 		scan->xs_recheckorderby = false;
 
