@@ -4,7 +4,7 @@
  *	  routines to manage scans of inverted index relations
  *
  *
- * Portions Copyright (c) 2015-2016, Postgres Professional
+ * Portions Copyright (c) 2015-2019, Postgres Professional
  * Portions Copyright (c) 1996-2016, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
@@ -15,7 +15,6 @@
 
 #include "access/relscan.h"
 #include "pgstat.h"
-#include "utils/memutils.h"
 
 #include "rum.h"
 
@@ -42,6 +41,13 @@ rumbeginscan(Relation rel, int nkeys, int norderbys)
 
 	initRumState(&so->rumstate, scan->indexRelation);
 
+#if PG_VERSION_NUM >= 120000
+	/*
+	 * Starting from PG 12 we need to invalidate result's item pointer. Earlier
+	 * it was done by invalidating scan->xs_ctup by RelationGetIndexScan().
+	 */
+	ItemPointerSetInvalid(&scan->xs_heaptid);
+#endif
 	scan->opaque = so;
 
 	return scan;
