@@ -4,7 +4,7 @@
  *	  utilities routines for the postgres inverted index access method.
  *
  *
- * Portions Copyright (c) 2015-2016, Postgres Professional
+ * Portions Copyright (c) 2015-2019, Postgres Professional
  * Portions Copyright (c) 1996-2016, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
@@ -239,13 +239,20 @@ initRumState(RumState * state, Relation index)
 			if (OidIsValid(rumConfig->addInfoTypeOid))
 				elog(ERROR, "AddTo could should not have AddInfo");
 
+			if (state->useAlternativeOrder && origAddAttr->attbyval == false)
+				elog(ERROR, "doesn't support order index over pass-by-reference column");
+
 			rumConfig->addInfoTypeOid = origAddAttr->atttypid;
 		}
 
 		if (state->oneCol)
 		{
 			state->tupdesc[i] = CreateTemplateTupleDesc(
+#if PG_VERSION_NUM >= 120000
+						OidIsValid(rumConfig->addInfoTypeOid) ? 2 : 1);
+#else
 						OidIsValid(rumConfig->addInfoTypeOid) ? 2 : 1, false);
+#endif
 			TupleDescInitEntry(state->tupdesc[i], (AttrNumber) 1, NULL,
 							   origAttr->atttypid,
 							   origAttr->atttypmod,
@@ -266,7 +273,11 @@ initRumState(RumState * state, Relation index)
 		else
 		{
 			state->tupdesc[i] = CreateTemplateTupleDesc(
+#if PG_VERSION_NUM >= 120000
+						OidIsValid(rumConfig->addInfoTypeOid) ? 3 : 2);
+#else
 						OidIsValid(rumConfig->addInfoTypeOid) ? 3 : 2, false);
+#endif
 			TupleDescInitEntry(state->tupdesc[i], (AttrNumber) 1, NULL,
 							   INT2OID, -1, 0);
 			TupleDescInitEntry(state->tupdesc[i], (AttrNumber) 2, NULL,
@@ -557,6 +568,7 @@ RumInitPage(Page page, uint32 f, Size pageSize)
 	opaque->flags = f;
 	opaque->leftlink = InvalidBlockNumber;
 	opaque->rightlink = InvalidBlockNumber;
+	RumItemSetMin(RumDataPageGetRightBound(page));
 }
 
 void
@@ -1045,8 +1057,40 @@ FunctionCall10Coll(FmgrInfo *flinfo, Oid collation, Datum arg1, Datum arg2,
 				   Datum arg6, Datum arg7, Datum arg8,
 				   Datum arg9, Datum arg10)
 {
-	FunctionCallInfoData fcinfo;
 	Datum		result;
+#if PG_VERSION_NUM >= 120000
+	LOCAL_FCINFO(fcinfo, 10);
+
+	InitFunctionCallInfoData(*fcinfo, flinfo, 10, collation, NULL, NULL);
+
+	fcinfo->args[0].value = arg1;
+	fcinfo->args[0].isnull = false;
+	fcinfo->args[1].value = arg2;
+	fcinfo->args[1].isnull = false;
+	fcinfo->args[2].value = arg3;
+	fcinfo->args[2].isnull = false;
+	fcinfo->args[3].value = arg4;
+	fcinfo->args[3].isnull = false;
+	fcinfo->args[4].value = arg5;
+	fcinfo->args[4].isnull = false;
+	fcinfo->args[5].value = arg6;
+	fcinfo->args[5].isnull = false;
+	fcinfo->args[6].value = arg7;
+	fcinfo->args[6].isnull = false;
+	fcinfo->args[7].value = arg8;
+	fcinfo->args[7].isnull = false;
+	fcinfo->args[8].value = arg9;
+	fcinfo->args[8].isnull = false;
+	fcinfo->args[9].value = arg10;
+	fcinfo->args[9].isnull = false;
+
+	result = FunctionCallInvoke(fcinfo);
+
+	/* Check for null result, since caller is clearly not expecting one */
+	if (fcinfo->isnull)
+		elog(ERROR, "function %u returned NULL", fcinfo->flinfo->fn_oid);
+#else
+	FunctionCallInfoData fcinfo;
 
 	InitFunctionCallInfoData(fcinfo, flinfo, 10, collation, NULL, NULL);
 
@@ -1076,6 +1120,7 @@ FunctionCall10Coll(FmgrInfo *flinfo, Oid collation, Datum arg1, Datum arg2,
 	/* Check for null result, since caller is clearly not expecting one */
 	if (fcinfo.isnull)
 		elog(ERROR, "function %u returned NULL", fcinfo.flinfo->fn_oid);
+#endif
 
 	return result;
 }
